package com.cnbm.packing.controller;

import com.cnbm.admin.annotation.LogOperation;
import com.cnbm.common.constant.Constant;
import com.cnbm.common.page.PageData;
import com.cnbm.common.utils.ExcelUtils;
import com.cnbm.common.utils.Result;
import com.cnbm.common.validator.AssertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.common.validator.group.UpdateGroup;
import com.cnbm.packing.dto.PrintModelDTO;
import com.cnbm.packing.excel.PrintModelExcel;
import com.cnbm.packing.service.PrintModelServiceBiz;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;


/**
 * 打印标签模板表  前端控制器
 *
 * @author packingGenerator
 * @since  2023-02-15
 */
@RestController
@RequestMapping("/packing/printModel")
@Api(tags="打印标签模板表")
public class PrintModelController {
    @Autowired
    private PrintModelServiceBiz printModelService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataTypeClass=String.class) ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataTypeClass=String.class),
        @ApiImplicitParam(name = "name", value = "标签名称", paramType = "query", dataTypeClass = String.class),
        @ApiImplicitParam(name = "type", value = "类型，0：模组标签，1：等级标签", paramType = "query", dataTypeClass = Integer.class),
        @ApiImplicitParam(name = "lineBody", value = "线体，1=F ; 2=S", paramType = "query", dataTypeClass = Integer.class),
        @ApiImplicitParam(name = "isEnable", value = "启用状态:0 、停用，1、启用", paramType = "query", dataTypeClass = Integer.class)
    })
    //@PreAuthorize("@ex.hasAuthority('packing:printModel:page')")
    public Result<PageData<PrintModelDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<PrintModelDTO> page = printModelService.page(params);
        return new Result<PageData<PrintModelDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @PreAuthorize("@ex.hasAuthority('packing:printModel:info')")
    public Result<PrintModelDTO> get(@PathVariable("id") Long id){
        PrintModelDTO data = printModelService.get(id);

        return new Result<PrintModelDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @PreAuthorize("@ex.hasAuthority('packing:printModel:save')")
    public Result<Long> save(@RequestBody PrintModelDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        printModelService.save(dto);

        return new Result<Long>().ok(dto.getId());
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @PreAuthorize("@ex.hasAuthority('packing:printModel:update')")
    public Result<Long> update(@RequestBody PrintModelDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        printModelService.update(dto);

        return new Result<Long>().ok(dto.getId());
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @PreAuthorize("@ex.hasAuthority('packing:printModel:delete')")
    public Result delete(@RequestBody Long[] ids){
        //效验数据
        AssertUtils.isArrayEmpty(ids, "id");

        printModelService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @PreAuthorize("@ex.hasAuthority('packing:printModel:export')")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<PrintModelDTO> list = printModelService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, PrintModelExcel.class);
    }
    
    @PostMapping(value = "list")
    @ApiOperation(value = "获取打印标签模板列表")
    public List<PrintModelDTO> list() { return printModelService.list(); }

    @PostMapping(value = "getCode")
    @ApiOperation(value = "获取标签编码")
    public Result<String> getCode() {
        String code = printModelService.getCode();
        return new Result<String>().ok(code);
    }

    @GetMapping("getByLineBody/{lineBody}")
    @ApiOperation("依据line_body 查询，is_enable==1 && type==2  limit 1的数据")
    public Result<PrintModelDTO> getByLineBody(@PathVariable("lineBody") Integer lineBody){
        PrintModelDTO dto = printModelService.getByLineBody(lineBody);
        return new Result<PrintModelDTO>().ok(dto);
    }

}